# -*- coding: utf-8 -*-

REGIONI = {
    'Abruzzo': [
        {'abbr': 'AQ', 'nome': "L'Aquila"},
        {'abbr': 'TE', 'nome': 'Teramo'},
        {'abbr': 'PE', 'nome': 'Pescara'},
        {'abbr': 'CH', 'nome': 'Chieti'}
    ],
    'Basilicata': [
        {'abbr': 'PZ', 'nome': 'Potenza'},
        {'abbr': 'MT', 'nome': 'Matera'}
    ],
    'Calabria': [
        {'abbr': 'CS', 'nome': 'Cosenza'},
        {'abbr': 'CZ', 'nome': 'Catanzaro'},
        {'abbr': 'RC', 'nome': 'Reggio di Calabria'},
        {'abbr': 'KR', 'nome': 'Crotone'},
        {'abbr': 'VV', 'nome': 'Vibo Valentia'}
    ],
    'Campania': [
        {'abbr': 'CE', 'nome': 'Caserta'},
        {'abbr': 'BN', 'nome': 'Benevento'},
        {'abbr': 'NA', 'nome': 'Napoli'},
        {'abbr': 'AV', 'nome': 'Avellino'},
        {'abbr': 'SA', 'nome': 'Salerno'}
    ],
    'Emilia-Romagna': [
        {'abbr': 'PC', 'nome': 'Piacenza'},
        {'abbr': 'PR', 'nome': 'Parma'},
        {'abbr': 'RE', 'nome': "Reggio nell'Emilia"},
        {'abbr': 'MO', 'nome': 'Modena'},
        {'abbr': 'BO', 'nome': 'Bologna'},
        {'abbr': 'FE', 'nome': 'Ferrara'},
        {'abbr': 'RA', 'nome': 'Ravenna'},
        {'abbr': 'FC', 'nome': 'Forlì-Cesena'},
        {'abbr': 'RN', 'nome': 'Rimini'}
    ],
    'Friuli-Venezia Giulia': [
        {'abbr': 'UD', 'nome': 'Udine'},
        {'abbr': 'GO', 'nome': 'Gorizia'},
        {'abbr': 'TS', 'nome': 'Trieste'},
        {'abbr': 'PN', 'nome': 'Pordenone'}
    ],
    'Lazio': [
        {'abbr': 'VT', 'nome': 'Viterbo'},
        {'abbr': 'RI', 'nome': 'Rieti'},
        {'abbr': 'RM', 'nome': 'Roma'},
        {'abbr': 'LT', 'nome': 'Latina'},
        {'abbr': 'FR', 'nome': 'Frosinone'}
    ],
    'Liguria': [
        {'abbr': 'IM', 'nome': 'Imperia'},
        {'abbr': 'SV', 'nome': 'Savona'},
        {'abbr': 'GE', 'nome': 'Genova'},
        {'abbr': 'SP', 'nome': 'La Spezia'}
    ],
    'Lombardia': [
        {'abbr': 'VA', 'nome': 'Varese'},
        {'abbr': 'CO', 'nome': 'Como'},
        {'abbr': 'SO', 'nome': 'Sondrio'},
        {'abbr': 'MI', 'nome': 'Milano'},
        {'abbr': 'BG', 'nome': 'Bergamo'},
        {'abbr': 'BS', 'nome': 'Brescia'},
        {'abbr': 'PV', 'nome': 'Pavia'},
        {'abbr': 'CR', 'nome': 'Cremona'},
        {'abbr': 'MN', 'nome': 'Mantova'},
        {'abbr': 'LC', 'nome': 'Lecco'},
        {'abbr': 'LO', 'nome': 'Lodi'},
        {'abbr': 'MB', 'nome': 'Monza e della Brianza'}
    ],
    'Marche': [
        {'abbr': 'PU', 'nome': 'Pesaro e Urbino'},
        {'abbr': 'AN', 'nome': 'Ancona'},
        {'abbr': 'MC', 'nome': 'Macerata'},
        {'abbr': 'AP', 'nome': 'Ascoli Piceno'},
        {'abbr': 'FM', 'nome': 'Fermo'}
    ],
    'Molise': [
        {'abbr': 'CB', 'nome': 'Campobasso'},
        {'abbr': 'IS', 'nome': 'Isernia'}
    ],
    'Piemonte': [
        {'abbr': 'TO', 'nome': 'Torino'},
        {'abbr': 'VC', 'nome': 'Vercelli'},
        {'abbr': 'NO', 'nome': 'Novara'},
        {'abbr': 'CN', 'nome': 'Cuneo'},
        {'abbr': 'AT', 'nome': 'Asti'},
        {'abbr': 'AL', 'nome': 'Alessandria'},
        {'abbr': 'BI', 'nome': 'Biella'},
        {'abbr': 'VB', 'nome': 'Verbano-Cusio-Ossola'}
    ],
    'Puglia': [
        {'abbr': 'FG', 'nome': 'Foggia'},
        {'abbr': 'BA', 'nome': 'Bari'},
        {'abbr': 'TA', 'nome': 'Taranto'},
        {'abbr': 'BR', 'nome': 'Brindisi'},
        {'abbr': 'LE', 'nome': 'Lecce'},
        {'abbr': 'BT', 'nome': 'Barletta-Andria-Trani'}
    ],
    'Sardegna': [
        {'abbr': 'SS', 'nome': 'Sassari'},
        {'abbr': 'NU', 'nome': 'Nuoro'},
        {'abbr': 'CA', 'nome': 'Cagliari'},
        {'abbr': 'OR', 'nome': 'Oristano'},
        {'abbr': 'OT', 'nome': 'Olbia-Tempio'},
        {'abbr': 'OG', 'nome': 'Ogliastra'},
        {'abbr': 'VS', 'nome': 'Medio Campidano'},
        {'abbr': 'CI', 'nome': 'Carbonia-Iglesias'}
    ],
    'Sicilia': [
        {'abbr': 'TP', 'nome': 'Trapani'},
        {'abbr': 'PA', 'nome': 'Palermo'},
        {'abbr': 'ME', 'nome': 'Messina'},
        {'abbr': 'AG', 'nome': 'Agrigento'},
        {'abbr': 'CL', 'nome': 'Caltanissetta'},
        {'abbr': 'EN', 'nome': 'Enna'},
        {'abbr': 'CT', 'nome': 'Catania'},
        {'abbr': 'RG', 'nome': 'Ragusa'},
        {'abbr': 'SR', 'nome': 'Siracusa'}
    ],
    'Toscana': [
        {'abbr': 'MS', 'nome': 'Massa-Carrara'},
        {'abbr': 'LU', 'nome': 'Lucca'},
        {'abbr': 'PT', 'nome': 'Pistoia'},
        {'abbr': 'FI', 'nome': 'Firenze'},
        {'abbr': 'LI', 'nome': 'Livorno'},
        {'abbr': 'PI', 'nome': 'Pisa'},
        {'abbr': 'AR', 'nome': 'Arezzo'},
        {'abbr': 'SI', 'nome': 'Siena'},
        {'abbr': 'GR', 'nome': 'Grosseto'},
        {'abbr': 'PO', 'nome': 'Prato'}
    ],
    'Trentino-Alto Adige/Südtirol': [
        {'abbr': 'BZ', 'nome': 'Bolzano/Bozen'},
        {'abbr': 'TN', 'nome': 'Trento'}
    ],
    'Umbria': [
        {'abbr': 'PG', 'nome': 'Perugia'},
        {'abbr': 'TR', 'nome': 'Terni'}
    ],
    "Valle d'Aosta/Vallée d'Aoste": [
        {'abbr': 'AO', 'nome': "Valle d'Aosta/Vallée d'Aoste"}
    ],
    'Veneto': [
        {'abbr': 'VR', 'nome': 'Verona'},
        {'abbr': 'VI', 'nome': 'Vicenza'},
        {'abbr': 'BL', 'nome': 'Belluno'},
        {'abbr': 'TV', 'nome': 'Treviso'},
        {'abbr': 'VE', 'nome': 'Venezia'},
        {'abbr': 'PD', 'nome': 'Padova'},
        {'abbr': 'RO', 'nome': 'Rovigo'}
    ]
}



from django.core.management.base import BaseCommand, CommandError
import csv
from django.utils.six import PY3
from italian_utils.models import Comune


class Command(BaseCommand):
    help = "Importa l'elenco dei comuni proveniente dal sito istat"

    def add_arguments(self, parser):
        parser.add_argument('csv_file')

    def handle(self, *args, **options):
        with open(options['csv_file'], 'rb') as csvfile:
            csvstring = csvfile.read()
        if PY3:
            import io
            csvfile = io.TextIOWrapper(
                io.BytesIO(csvstring),
                encoding="latin-1"
            )
        else:
            import StringIO
            csvfile = StringIO.StringIO(csvstring)

        comuni_reader = csv.DictReader(csvfile, delimiter=';')
        for row in comuni_reader:
            nome = row['Denominazione (Italiana e straniera)']
            codice_catastale = row['Codice Catastale del comune']
            if nome and codice_catastale:
                c = Comune(nome=nome, codice_catastale=codice_catastale)
                c.save()
        self.stdout.write('Comuni importati correttamente')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('italian_utils', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='comune',
            options={'verbose_name': 'comune', 'verbose_name_plural': 'comuni'},
        ),
        migrations.AlterField(
            model_name='comune',
            name='codice_catastale',
            field=models.CharField(max_length=4, unique=True),
        ),
    ]



from django.test import TestCase
from django.core.exceptions import ValidationError
from django.core.management.base import CommandError
from italian_utils.validators import validate_codice_fiscale
from italian_utils.validators import validate_partita_iva
from italian_utils.utils import REGIONI
from italian_utils.models import Comune
from django.core.management import call_command


class ValidateCodiceFiscaleTestCase(TestCase):
    def test_codice_fiscale_vuoto(self):
        self.assertRaises(
            ValidationError,
            validate_codice_fiscale,
            ''
        )

    def test_codice_fiscale_16(self):
        self.assertRaises(
            ValidationError,
            validate_codice_fiscale,
            '1234567890123456'
        )

    def test_codice_fiscale_controllo(self):
        self.assertRaises(
            ValidationError,
            validate_codice_fiscale,
            'ABCDEF00A00A000A'
        )

    def test_codice_fiscale_formalmente_corretto(self):
        self.assertEqual(validate_codice_fiscale('RSSMRA14M26H501N'), None)


class ValidatePartitaIva(TestCase):
    def test_partita_iva_vuota(self):
        self.assertRaises(
            ValidationError,
            validate_partita_iva,
            ''
        )

    def test_partita_iva_non_numerica(self):
        self.assertRaises(
            ValidationError,
            validate_partita_iva,
            'ABC123DEF45'
        )

    def test_partita_iva_controllo(self):
        self.assertRaises(
            ValidationError,
            validate_partita_iva,
            '12345678901'
        )

    def test_partita_iva_formalmente_corretta(self):
        self.assertEqual(validate_partita_iva('12345678905'), None)


class ValidateRegioni(TestCase):
    def test_numero_regioni(self):
        self.assertEqual(len(REGIONI), 20)

    def test_roma_in_lazio(self):
        self.assertTrue(
            any(['Roma' in comune['nome'] for comune in REGIONI['Lazio']])
        )


class ValidateComuni(TestCase):
    def test_crea_comune(self):
        comune = Comune(nome='Catania', codice_catastale='C351')
        self.assertEqual(str(comune), 'C351: Catania')


class ValidateImportaComuni(TestCase):
    def test_importa_comuni_no_param(self):
        self.assertRaises(
            CommandError,
            call_command,
            'importacomuni'
        )

    def test_importa_comuni(self):
        import os
        filename = 'italian_utils/tests/comuni-italiani.csv'
        call_command('importacomuni', filename)



from django.core.exceptions import ValidationError
import re
import string


def validate_codice_fiscale(value):
    codice_fiscale_re = re.compile(
        r'^[a-zA-Z]{6}\d{2}[a-zA-Z]\d{2}[a-zA-Z]\d{3}[a-zA-Z]'
    )

    if not codice_fiscale_re.match(value):
        raise ValidationError('Inserisci un codice fiscale formalmente valido')

    conversione_pari = {
        '0': 0,
        '1': 1,
        '2': 2,
        '3': 3,
        '4': 4,
        '5': 5,
        '6': 6,
        '7': 7,
        '8': 8,
        '9': 9,
        'A': 0,
        'B': 1,
        'C': 2,
        'D': 3,
        'E': 4,
        'F': 5,
        'G': 6,
        'H': 7,
        'I': 8,
        'J': 9,
        'K': 10,
        'L': 11,
        'M': 12,
        'N': 13,
        'O': 14,
        'P': 15,
        'Q': 16,
        'R': 17,
        'S': 18,
        'T': 19,
        'U': 20,
        'V': 21,
        'W': 22,
        'X': 23,
        'Y': 24,
        'Z': 25,
    }

    conversione_dispari = {
        '0': 1,
        '1': 0,
        '2': 5,
        '3': 7,
        '4': 9,
        '5': 13,
        '6': 15,
        '7': 17,
        '8': 19,
        '9': 21,
        'A': 1,
        'B': 0,
        'C': 5,
        'D': 7,
        'E': 9,
        'F': 13,
        'G': 15,
        'H': 17,
        'I': 19,
        'J': 21,
        'K': 2,
        'L': 4,
        'M': 18,
        'N': 20,
        'O': 11,
        'P': 3,
        'Q': 6,
        'R': 8,
        'S': 12,
        'T': 14,
        'U': 16,
        'V': 10,
        'W': 22,
        'X': 25,
        'Y': 24,
        'Z': 23,
    }

    conversione_controllo = dict(enumerate(string.ascii_uppercase))

    pari, dispari = [], []

    for i, x in enumerate(value[:-1].upper()):
        if i % 2 == 0:
            dispari.append(x)
        else:
            pari.append(x)

    valore_controllo = sum(
        [conversione_pari[x] for x in pari] +
        [conversione_dispari[x] for x in dispari]
    ) % 26

    carattere_controllo = conversione_controllo[valore_controllo]

    if carattere_controllo != value[-1].upper():
        raise ValidationError('Inserisci un codice fiscale formalmente valido')


def validate_partita_iva(value):
    partita_iva_re = re.compile(
        r'^\d{11}'
    )

    if not partita_iva_re.match(value):
        raise ValidationError('Inserisci una partita iva valida')

    pari, dispari = [], []

    for i, x in enumerate(value):
        if i % 2 == 0:
            dispari.append(int(x))
        else:
            pari.append(int(x))

    controllo = (sum(pari) + sum(dispari)) % 10
    if controllo != 0:
        raise ValidationError('Inserisci una partita iva valida')



import os
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='django-italian-utils',
    version='0.3.1',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description='Libreria di utility per semplificare la creazione di applicazioni italiane.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/facciocose/django-italian-utils',
    author='Luca Marra',
    author_email='luca@facciocose.it',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class Comune(models.Model):
    nome = models.CharField(max_length=255)
    codice_catastale = models.CharField(max_length=4, unique=True)

    class Meta:
        verbose_name = 'comune'
        verbose_name_plural = 'comuni'

    def __str__(self):
        return "%s: %s" % (self.codice_catastale, self.nome)
